package com.mt.wms.empty.controller;

import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.OrderInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;

@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "apms")
@Slf4j
@Api(value = "Apms相关接口", tags = "Apms相关接口", hidden = false)
public class ApmsController extends BaseController {
    @Autowired
    private OrderInfoService orderInfoService;

    static String Token = "";
    static String ApiAddress = "http://59.110.171.25:9010";

    @PostMapping(value = "apmsPostOrder")
    @ApiOperation(value = "接收APMS推送订单信息")
    public R apmsPostOrder(@Validated @RequestBody OrderParamForApms apmsPostOrderParam) {
        return orderInfoService.apmsPostOrder(apmsPostOrderParam);
    }

    @PostMapping(value = "createProcessSheet")
    @ApiOperation(value = "通知APMS开始加工处理信息")
    public R createProcessSheet(@Validated @RequestBody ApmsCreateProcessSheet apmsCreateProcess) {
        //todo
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsCreateProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/createProcessSheet")
                .header("token", Token).form(paramMap).execute();
        return successful(response);
    }

    @PostMapping(value = "startProcess")
    @ApiOperation(value = "通知APMS订单开始处理")
    public R startProcess(@Validated @RequestBody ApmsStartProcess apmsStartProcess) {
        //todo
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsStartProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/startProcess")
                .header("token", Token).form(paramMap).execute();
        return successful(response);
    }

    @PostMapping(value = "endProcess")
    @ApiOperation(value = "通知APMS订单结束处理")
    public R endProcess(@Validated @RequestBody ApmsEndProcess apmsEndProcess) {
        //todo
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsEndProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/endProcess")
                .header("token", Token).form(paramMap).execute();
        return successful(response);
    }

    @PostMapping(value = "finishProcessSheet")
    @ApiOperation(value = "通知APMS结束加工处理信息")
    public R finishProcessSheet(@Validated @RequestBody ApmsFinishProcessSheet apmsFinishProcessSheet) {
        //todo
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsFinishProcessSheet);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/finishProcessSheet")
                .header("token", Token).form(paramMap).execute();
        return successful(response);
    }

    @PostMapping(value = "getApmsToken")
    @ApiOperation(value = "获取APMS获得Token")
    public R getApmsToken() {
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("apiKey", "7ee8f59bbaeae27e");
        paramMap.put("platFormCode", "CUC");
        String result = HttpUtil.get(ApiAddress + "/platform/api/getToken", paramMap);
        JSON parse = JSONUtil.parse(result);
        Object token = parse.getByPath("token");
        Token = token.toString();
        //todo 记录通讯日志
        return successful(result);
    }

    @PostMapping(value = "getStoveCodeByWorkShopCode")
    @ApiOperation(value = "APMS通过车间编码获取炉号信息")
    public R getStoveCodeByWorkShopCode(String workShopCode) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("workShopCode", workShopCode);
        HttpResponse response = HttpUtil.createGet(ApiAddress + "/platform/api/getStoveCodeByWorkShopCode")
                .header("token", Token).form(paramMap).execute();
        //todo 记录通讯日志
        return successful(response);
    }

    private void checkToken() {
        if (Token.isEmpty())
        {
            getApmsToken();
        }
    }
}
