package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.CurrTask;
import com.mt.wms.core.dal.entity.CurrTaskDet;
import com.mt.wms.core.dal.entity.TaskDetHis;
import com.mt.wms.core.dal.entity.TaskHis;
import com.mt.wms.core.dal.service.CurrTaskServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.enums.TaskTypeEnum;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.TaskDetHisService;
import com.mt.wms.empty.service.TaskHisService;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import com.mt.wms.empty.vo.CurrTaskVo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class CurrTaskServiceImpl extends BaseService implements CurrTaskService {

    @Autowired
    CurrTaskServiceBiz currTaskServiceBiz;

    @Autowired
    CurrTaskDetService currTaskDetService;

    @Autowired
    TaskHisService taskHisService;

    @Autowired
    TaskDetHisService taskDetHisService;

    @Override
    public CurrTaskVo getCurrTask(IdParam idParam) {
        CurrTask byId = currTaskServiceBiz.getById(idParam.getId());
        CurrTaskVo build = CurrTaskVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        return build;
    }

    @Override
    public R<IdVo> createProcessTask(TaskCreateParam param) {
        CurrTask currTask = new CurrTask();
        BeanUtils.copyProperties(param, currTask);
        currTask.setTaskType(TaskTypeEnum.WTK.getValue());
        currTask.setTaskSource(0);
        setCommonField(currTask);
        currTaskServiceBiz.save(currTask);
        return successful(IdVo.builder().id(currTask.getId()).build());
    }

    @Override
    public R<PageVo<CurrTaskQueryVo>> currentTaskPage(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(StringUtils.isNotBlank(param.getVehicleId()), CurrTask.VEHICLE_ID, param.getVehicleId());
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        return successful(new PageVo<>(page, CurrTaskQueryVo.class));
    }

    @Override
    public R completeTask(IdParam param) {
        CurrTask byId = currTaskServiceBiz.getById(param.getId());
        TaskHis taskHis = new TaskHis();
        BeanUtils.copyProperties(byId, taskHis);
        taskHis.setStatus(2);
        taskHis.setId(null);
        //将主任务放入到历史表，清空id，防止重复
        R<IdVo> save = taskHisService.save(taskHis);
        currTaskServiceBiz.removeById(byId.getId());
        List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(byId.getId()).build());
        List<TaskDetHis> taskDetHis = com.mt.wms.core.utils.BeanUtils.copyList(taskDetailsByMainId, TaskDetHis.class);
        taskDetHis.forEach(detHis -> {
            detHis.setTaskId(save.getData().getId());
        });
        //将详细任务信息放入详细历史表
        taskDetHisService.saveList(taskDetHis);
        return successful("执行成功。");
    }
}
