package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.KilnInfoQueryParam;
import com.mt.wms.basic.params.VehicleQueryParam;
import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.service.LocationInfoService;
import com.mt.wms.basic.service.VehicleService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.OrderInfo;
import com.mt.wms.core.dal.service.OrderInfoServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.enums.OrderSourceEnum;
import com.mt.wms.empty.params.OrderInfoBasePageParam;
import com.mt.wms.empty.params.OrderInfoPersonCreateParam;
import com.mt.wms.empty.params.OrderInfoTaskDetParam;
import com.mt.wms.empty.params.OrderParamForApms;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.OrderInfoHisService;
import com.mt.wms.empty.service.OrderInfoService;
import com.mt.wms.empty.vo.CurrTaskVo;
import com.mt.wms.empty.vo.OrderInfoTaskDetVo;
import com.mt.wms.empty.vo.OrderInfoVo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class OrderInfoServiceImpl extends BaseService implements OrderInfoService {

    @Autowired
    OrderInfoServiceBiz orderInfoService;

    @Autowired
    OrderInfoHisService orderInfoHisService;
    @Autowired
    CurrTaskDetService currTaskDetService;
    @Autowired
    CurrTaskService currTaskService;
    @Autowired
    KilnInfoService kilnInfoService;

    @Autowired
    VehicleService vehicleService;

    @Autowired
    LocationInfoService locationInfoService;


    @Override
    public R apmsPostOrder(OrderParamForApms apmsPostOrderParam) {
        OrderInfo orderInfo = new OrderInfo();
        BeanUtils.copyProperties(apmsPostOrderParam,orderInfo);
        setCommonField(orderInfo);
        //设定来源信息为apms
        orderInfo.setOrderSource(OrderSourceEnum.APMS.getValue());
        orderInfo.setInterCode("APMS"+System.currentTimeMillis());
        orderInfoService.save(orderInfo);
        //todo 记录通讯日志信息
        return successful("Wms接收Apms订单成功。");
    }

    @Override
    public R<PageVo<OrderInfoVo>> page(OrderInfoBasePageParam param) {
        QueryWrapper<OrderInfo> wrapper=new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(param.getIdenCardNum()),OrderInfo.IDEN_CARD_NUM, param.getIdenCardNum())
                .like(StringUtils.isNotBlank(param.getOrderNo()),OrderInfo.ORDER_NO, param.getOrderNo())
                .eq(OrderInfo.VALID,1)
                .orderByDesc(OrderInfo.CREATE_TIME);
        Page<OrderInfo> page = orderInfoService.page(new Page<>(param.getCurrent(), param.getSize()), wrapper);
        return successful(new PageVo<>(page,OrderInfoVo.class));
    }

    @Override
    public R personCreateOrder(OrderInfoPersonCreateParam orderInfoPersonCreateParam) {
        OrderInfo orderInfo = new OrderInfo();
        BeanUtils.copyProperties(orderInfoPersonCreateParam,orderInfo);
        setCommonField(orderInfo);
        //设定来源信息为人工
        orderInfo.setOrderSource(OrderSourceEnum.PERSON.getValue());
        orderInfo.setInterCode("PERSONAL"+System.currentTimeMillis());
        orderInfoService.save(orderInfo);
        //todo 通知apms创建了一个订单
        return successful("人工创建订单成功。");
    }

    @Override
    public R<OrderInfoVo> getOne(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        OrderInfoVo build = OrderInfoVo.builder().build();
        BeanUtils.copyProperties(byId,build);
        return successful(build);
    }


    /**
     * 获取详细的拆分任务信息
      */
    @Override
    public R<PageVo<OrderInfoTaskDetVo>> getTaskInfo(OrderInfoTaskDetParam param) {
        //从任务详细中获取该标识卡和订单的所有订单
        PageVo<OrderInfoTaskDetVo> currOrderCurrTask = currTaskDetService.getCurrOrderCurrTask(param);
        Assert.gtZero(currOrderCurrTask.getRecords().size(),"该订单暂无详细记录。");
        //获取车辆信息
        R<List<VehicleVo>> vehicle = vehicleService.list(new VehicleQueryParam());
        Map<Long, List<VehicleVo>> collect = vehicle.getData().stream().collect(Collectors.groupingBy(VehicleVo::getId));
        //获取窑炉信息
        R<List<KilnInfoVo>> kiln = kilnInfoService.list(new KilnInfoQueryParam());
        Map<Long, List<KilnInfoVo>> collect2 = kiln.getData().stream().collect(Collectors.groupingBy(KilnInfoVo::getId));
        currOrderCurrTask.getRecords().forEach(orderInfo ->
                {
                    //获取对应的任务
                    CurrTaskVo currTask = currTaskService.getCurrTask(IdParam.builder().id(orderInfo.getCurrTaskId()).build());
                    orderInfo.setABeginTime(currTask.getABeginTime());
                    orderInfo.setAEndTime(currTask.getAEndTime());
                    orderInfo.setTaskType(currTask.getTaskType());
                    orderInfo.setStatus(currTask.getStatus());
                    orderInfo.setKilnId(currTask.getKilnId());
                    orderInfo.setStartPosition(currTask.getStartPosition());
                    orderInfo.setTargetPosition(currTask.getTargetPosition());
                    orderInfo.setVehicleId(currTask.getVehicleId());
                    //获取窑炉和车辆的名称信息
                    if (orderInfo.getKilnId() != null)
                    {
                        orderInfo.setKilnName(collect2.get(orderInfo.getKilnId()).get(0).getKilnName());
                    }
                    if (orderInfo.getVehicleId() != null)
                    {
                        orderInfo.setVehicleName(collect.get(orderInfo.getVehicleId()).get(0).getVehicleName());
                    }
                }
        );
        return successful(currOrderCurrTask);
    }

    @Override
    public R<OrderInfoVo> getTaskInfoByIdenCardNum(String idenCardNum) {
        QueryWrapper<OrderInfo> wrapper = new QueryWrapper<>();
        wrapper.eq(OrderInfo.IDEN_CARD_NUM, idenCardNum).eq(OrderInfo.VALID, 1).eq(OrderInfo.STATUS, 0);
        OrderInfo one = orderInfoService.getOne(wrapper);
        OrderInfoVo build = OrderInfoVo.builder().build();
        BeanUtils.copyProperties(one, build);
        return successful(build);
    }

    @Override
    public R completeOrder(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        orderInfoHisService.save(byId);
        byId.setValid(0);
        byId.setStatus(2);
        orderInfoService.updateById(byId);
        return successful("操作成功。");
    }

    @Override
    public R voidOrder(IdParam idParam) {
        OrderInfo byId = orderInfoService.getById(idParam.getId());
        byId.setValid(0);
        orderInfoService.updateById(byId);
        return successful("操作成功。");
    }
}
